function [wfm, time, header] = getWfm(com, channel) 
  narginchk(2, 2);
  nargoutchk(0, 3);
  
  % channels = [varargin{:}]
  
  if ~isscalar(channel)
    error('channels has to be a scalar number');
  end
  
  %% setup
  %fprintf(com, 'FORM REAL,32'); [YS] Cut to main program
  sampleSize = 4;

%   fprintf(RTx, 'FORM:BORD LSBF');     % little endian
%   fprintf(RTx, 'SYST:DISP:UPD OFF');  % display off
%   fprintf(RTx, 'EXP:WAV:FAST ON');    % fast export

  %% get header
  if nargout >= 2
    fprintf(com, 'CHAN%d:DATA:HEAD?', channel);
    answer = fscanf(com, '%f,');
    header.start            = answer(1);
    header.stop             = answer(2);
    header.nSamples         = answer(3);
    header.valuesPerSample  = answer(4);
    
    time      = linspace(header.start, header.stop, header.nSamples);
    timeDiff  = header.stop - header.start;
    header.fs = header.nSamples / timeDiff;
  end

  %% get wfm

  %start = tic; don't necc. need it here

  fprintf(com, 'CHAN%d:DATA?', channel);

  % "#" expected at beginning of stream
  c = fscanf(com, '%c', 1);
  if ~strcmp(c, '#')
    error('Error reading waveform data.');
  end

  % Read length of data part from block-header
  nDigits   = str2double(fscanf(com, '%c', 1)); % read number of length digits
  nBytes    = str2double(fscanf(com, '%c', nDigits)); % read number of bytes
  nSamples  = nBytes / sampleSize;

  wfmData   = zeros(1, nSamples);
  nReceived = 0;

  try
    while (nReceived < nSamples)
      nSamplesLeft = nSamples - nReceived;
      if (nSamplesLeft * sampleSize > com.InputBufferSize)
        [tmp, cnt] = fread(com, SampleBufferSize, 'float32');
%         fprintf('%d samples read.\n', (nReceived + cnt));
      else
        [tmp, cnt] = fread(com, nSamplesLeft, 'float32'); 
        nSamplesLeft = 0;     %#ok, mlint
%         fprintf('%d samples read.\n', (nReceived + cnt));
      end
      wfmData((1:length(tmp)) + nReceived) = tmp;
      nReceived = nReceived + length(tmp);
    end
  catch err
    fprintf('Error reading waveform data: %s\n', err.identifier);
    rethrow(err);
  end

  %fprintf('transferred wfm data in %.0f ms\n')  [YS]toc(start)*1e3); % don't
  

  fscanf(com, '%c', 1); % EOL
  
  wfm = wfmData;
  
end
