% ***************************************************************************************************************
%   Rohde & Schwarz GmbH & Co. KG
% 2017/05 - Yariv Shavit, Alexander Naehring 1MAA.
% 
% How to use this project:
%   1.) Change the target IP address of the RTB in the code
%   2.) press Run 

%*****************************************************************************************************************

format compact;
%clear; close all; clc;
f = figure('Color', [0 0 0]);

%% general parameter
hostname       = '10.85.0.147';
vcoTableFile      = 'E:\VCO_TuningTable.txt'; %

%% Parameter for Display FFT
rangeOffset       = 2; % start offset of RTI in meter
maxRange          = 60; %  distance in [m] to be displayed
fromto_dB        = [0, 45]; % [YS] Dynamic range indication for RTI & FFT plots
MinPeakLimit = -6;  %Threshold to display Peak value in FFT-Plot
MinPeakDistance=6;
%% parameter for color Display RTI & Clutter rejection
linesToDisplay    = 25; % Y-Axis (history in RTI)
map               = colormap('hot');
map(1:20, :) = 0; % Set the first 20 colors (=small values) to black.
clutterRejection  = 4; % number of recent captures to use
%% connect
com = openDevice(hostname); % function that opens a VISA session
fprintf(com, sprintf('WGENerator:OUTPut ON;*OPC?\n')); %generator Switch On  
fprintf(com, sprintf('FORM:BORD LSBF;:FORM REAL,32\n'));
fprintf(com, sprintf('SING;*OPC?\n'));
status = fscanf(com, '%s');

%% get initial  Sawtooth (CH3) and signals IF signal (CH1)
[vChirp, vTime, header] = getWfm(com, 3);
[vSignal]               = getWfm(com, 1);
nSamples = header.nSamples;
fs = header.fs;
fprintf('fs = %.2f MHz\n', fs / 1e6);

%% actual Sawtooth Generator values transferred to frequencies applied to the VCO to calculate the bandwidth 
VCO_table = load(vcoTableFile); % [YS] loads the VCO tuning table Voltage to Frequency
vTime = vTime * 1e3;
V_max = round(max(vChirp)); % [YS] the V-max of Generator from channel 3
V_min = round(min(vChirp)); % [YS] the V-min of Generator from channel 3

%********* [YS] retrieves the frequency values of VCO according to Generator Voltage
[row] = find(VCO_table==V_min); %[YS] compared V(max/min) with VCO table
fStart = VCO_table(row,2)*10e5; %[YS] start freq of VCO  

[row]=find(VCO_table==V_max);
fStop= VCO_table(row,2)*10e5;   % [YS] Stop freq. of VCO


%% range resolution of fft bin
c         = 299792458;
BW        = fStop - fStart; % Bandwidth of chirp
rangeRes  = c/(2*BW); % in [m]

%% detect edge samples ([YS] edge is the largest difference beween min & max)
% no need to transfer chrip and calculate trigger again, as long as trigger
% signal is constant.
[first, last] = detectTrigger(vChirp);

%% initialize plots
fftLength = 1*ceil(maxRange/(rangeRes)); % X-Axis 
data      = zeros(linesToDisplay, fftLength); % [YS] Matrix Initializing a 2-D "data" matrix Lines x range [m]
dataCR    = data; %[Ys] Matrix Initializing a 2-D for clutter Rejection
line      = 0;
lastS     = [];
TimeOfChirp=(last-first)/fs; %[YS]reveals the time in [s]

% Top Left time domain
subplot(2, 2, 1);
grid on;
hold on;
handle = gca;
set(handle, 'Color', [0 0 0]);
set(handle, 'XColor', [1 1 1]);
set(handle, 'YColor', [1 1 1]);
set(handle, 'YTick', -.5:.1:.5);
ylim([-.6, .6]); %Set or  y-axis limits& removes autoscaling
xlabel('Time [ms]');
ylabel('');
title('\color{white}Time Domain');


% Top Right Freq Domain
subplot(2, 2, 2);
grid on;
hold on;
handle = gca;
set(handle, 'Color', [0 0 0]);
set(handle, 'XColor', [1 1 1]);
set(handle, 'YColor', [1 1 1]);
xlabel('Range [m] ');
ylabel('power');
xlim([rangeOffset maxRange]);
ylim(fromto_dB); %[YS] limit lines for Y axis
title('\color{white}Frequency Domain');
[len]=size(data);


% Colored Range Time Indication Window
subplot(2, 2, 4);
grid on;
hold on;
handle = gca;
set(handle, 'XColor', [1 1 1]);
set(handle, 'YColor', [1 1 1]);
set(handle, 'YDir', 'reverse');
%ylim([0 RTI_end]);
ylim([-linesToDisplay 0]); 
xlim([rangeOffset maxRange]);
xlabel('Range [m]');
ylabel('Time passed ');
title('\color{white}RTI');
colormap(gca, map);
colorbar;

% Colored Range Time Indication Window (ClusterRejection)
subplot(2, 2, 3);
grid on;
hold on;
handle = gca;
set(handle, 'XColor', [1 1 1]);
set(handle, 'YColor', [1 1 1]);
set(handle, 'YDir', 'reverse');
ylim([-linesToDisplay 0]); 
xlim([rangeOffset maxRange]);
xlabel('Range [m]');
ylabel('Time passed ');
title('\color{white}RTI with Clutter Rejection');
colormap(gca, map);




while true
  % do measurement
  start = tic;
  fprintf(com, 'SING;*WAI');
  
  [vSignal]               = getWfm(com, 1);
  vSignal = vSignal - mean(vSignal);
  
  % cut out used signal: one chirp, from start to end sample (trigger)
  s = vSignal(first:last-1); %[YS] ...had to take out last outliner
  
  
  %% calculate Spectrum/FFT/ RTI
  rti = 20*log10(abs(fft(s)));
  if smoothing==true
  rti= smooth(rti,'lowess'); %[YS] added rtiSmoothed instead rti 
  end
  freq = 0:fs/length(rti):fs/2; % [YS]generate a frequency axis
  halflength_rti=(length(rti)/2)+1;   % [YS] turned out that the +1 was missing....
  spectrum= rti(1:halflength_rti);% [YS] Warning: Integer operands are required for colon operator when used as index 
 
  rti = rti(1:fftLength);
  % move all lines up by one and add new line at bottom
  data(1:end-1, :) = data(2:end, :);
  data(end, :) = rti;
 
  %% -----------------------plot spectrum ------------------------------------------------------
  subplot(2, 2, 2);
  cla;
  %plot(rti, 'y');
  [Peaks, Locs]=findpeaks(rti,'MINPEAKHEIGHT', MinPeakLimit,'MINPEAKDISTANCE',MinPeakDistance); %[YS] added peaksearch
  Locs=round(rangeRes.*Locs);
  plot(linspace(0, maxRange, fftLength),rti,'y');% [YS] original rti not smoothed
  %text(Locs(1:8)+.05,Peaks(1:8),num2str(Locs(1:8)),'color','white'); % [YS] Displaying first 8 peaks
  text(Locs+.05,Peaks,num2str(Locs),'color','white'); % [YS] Displaying first 8 peaks
  
%   %% --------------again with clutter rejection -----------------------------------------------
  s = rejectClutter(s, clutterRejection);
  rti = 20*log10(abs(fft(s))); %[YS] added rtiSmoothed instead rti
  rti = rti(1:fftLength);
  % store in RTI history
  dataCR(1:end-1, :) = dataCR(2:end, :);
  dataCR(end, :) = rti;
  
  %% ----------------plot time signal
  subplot(2, 2, 1);
  cla;
  plot(vTime, .5*vChirp./max(vChirp), 'r', 'MarkerSize', 1);
  plot(vTime, .5*vSignal./max(vSignal), 'y', 'MarkerSize', 1);
  plot(vTime(first:last), -.5*ones(1, last-first+1), 'g', 'MarkerSize', 10);
  xlim([vTime(1) vTime(end)]);
  
  %% -----------------plot history (RTI) (w/o clutter rejection)
  subplot(2, 2, 4);
  cla;
  imagesc(linspace(0, maxRange, fftLength), -linesToDisplay:0, data, fromto_dB);%, [YS] added fromto_dB intead [30, 30] limit;
  
  %% --------------plot history (RTI) (with clutter rejection)
  subplot(2, 2, 3);
  cla;
  imagesc(linspace(0, maxRange, fftLength), -linesToDisplay:0, dataCR, fromto_dB);% [YS] added fromto_dB intead [30, 30] limit;
  
  
  
  drawnow;
  if ~ishandle(f);
    break;
  end
  fprintf('All plots & calc conf %.0f ms\n', toc(start)*1e3);
end
